{{
**********************************
* File jvmArray: handling arrays *
**********************************
}}

{{
  JEM array storage.
  Arrays are stored on the heap in the following format:
    int class Object   'offset to class Object header
    int elements       'number of elements
    byte data[elements*typesize]  'the space occupied by the elements
  Note that single boolean, byte and char variables each occupy 2 bytes, just as short, int and reference.
  For arrays only the references to the arrays are stored in the object. The arrays themselves are objects
  on the heap.
}}


OBJ
  js: "jvmStack"
  jd: "jvmData"
  jh: "jvmHeap"
  
  
PUB jb_newarray | type,size,i,aref,class,len
{{ Meaning: Allocate new array. }}  
{{
  Description: A new array of a specific array type, capable of holding size elements, is allocated. The result
  is a reference to the new object. Allocation of an array large enough to contain size items of the specific
  array type is attempted and all elements of the array are initialized to 0.  
  size represents the number of elements in the new array and must be an integer. The result is stored with an
  internal code that indicates the type of array to allocate. Possible values for the type of array are as follows:
  T_BOOLEAN 4, T_CHAR 5, T_FLOAT 6, T_DOUBLE 7, T_BYTE 8, T_SHORT 9, T_INT 10, T_LONG 11
  Note:
  A NegativeArraySizeException is thrown if size is less than 0. 
  An OutOfMemoryError is thrown if there is not enough memory to allocate the array.
}}
{{
  JEM format: newarray,byte1
  byte1 identifies type: 0 for short and int, 1 for boolean, byte and char. 
}}
  len := js.pop 'pop length
  if len > 0
    type := jd.readJavaByte(jd.getJvmPC+1)
    size := len*(2-type)
    aref := jh.allocate(size + 4)
    if aref <> 0
      class := jd.readJavaIntBE(jd#JVM_OBJECT)
      jd.writeJavaIntBE(aref+0,class)
      jd.writeJavaIntBE(aref+2,len)
      i := 4
      repeat size 'clear array
        jd.writeJavaByte(aref+i,$00)
        i++
      js.push(aref) 'push reference
      jd.incJvmPC(2)
    else
      jd.setException
      js.push($0000) 'do we need return value ??
      jd.incJvmPC(2)
  else
    jd.setException
    js.push($0000)  'do we need return value ??
    jd.incJvmPC(2)
    
  
PUB jb_anewarray | type,size,i,aref,class,len
{{ Meaning: Allocate new array of objects. }}
{{
  Description: A new array of the indicated class type and capable of holding size elements is allocated.
  The result is a reference to the new object. Allocation of an array large enough to contain size elements
  of the given class type is attempted and all elements of the array are initialized to null.  
  size represents the number of elements in the new array and must be an integer. byte1 and byte2
  are used to construct an index into the constant pool of the current class. When the item at that index
  is resolved, the resulting entry must be a class. 
  Note:
  A NegativeArraySizeException is thrown if size is less than 0. 
  An OutOfMemoryError is thrown if there is not enough memory to allocate the array.  

  The anewarray instruction is used to create a single-dimension array. For example, the declaration
  new Thread[7] generates the following bytecode instructions:  
    bipush 7 
    anewarray <Class "java.lang.Thread">
  The anewarray instruction can also be used to create the outermost dimension of a multidimensional array. 
  for example, the array declaration new int[6][] generates the following bytecode instructions:  
    bipush 6 
    anewarray <Class "[I">
}}
{{
  JEM format: anewarray,byte1,byte2
  byte1 is always 0. byte2 identifies type: 0 for short, int and reference, 1 for boolean, byte and char.
  Since array of objects equals array of references, byte2 is always 0 too. 
}}
  len := js.pop 'pop length
  if len > 0
    type := jd.readJavaByte(jd.getJvmPC+2)
    size := len*(2-type)
    aref := jh.allocate(size + 4)
    if aref <> 0
      class := jd.readJavaIntBE(jd#JVM_OBJECT)
      jd.writeJavaIntBE(aref+0,class)
      jd.writeJavaIntBE(aref+2,len)
      i := 4
      repeat size 'clear array
        jd.writeJavaByte(aref+i,$00)
        i++
      js.push(aref) 'push reference
      jd.incJvmPC(3)
    else
      jd.setException
      js.push($0000) 'do we need return value ??
      jd.incJvmPC(3)
  else
    jd.setException
    js.push($0000)  'do we need return value ??
    jd.incJvmPC(3)
    

{{PUB jb_multianewarray}}
{{ Meaning: Allocate new multidimensional array. }}
{{
  Description: A new multidimensional array of a specific array type is allocated. The number of dimensions
  in the array is determined by sizeN. The value of sizeN represents the number of elements in the new array
  and must be an integer. byte1 and byte2 are used to construct an index in the constant pool of the current class.
  The item at that index is resolved and the resulting entry must be an array class of one or more dimensions.
  Note:
  A NegativeArraySizeException is thrown if sizeN is less than 0. 
  An OutOfMemoryError is thrown if there is not enough memory to allocate the array.
}}


PUB jb_arraylength | aref,len
{{ Meaning: Get length of array. }}
{{
  Description: The length of the array is determined and replaces aref on the top of the stack. 
  The aref must be a reference to an array object. 
  Note:
  A NullPointerException is thrown if the aref is null.
}}
  aref := js.pop 'get reference
  len := jd.readJavaIntBE(aref + 2)
  js.push(len)
  jd.incJvmPC(1)

  
PUB jb_iaload | index,aref,val
{{ Meaning: Load integer from array. }}
{{
  Description: The integer value at the array aref is retrieved and pushed onto the top of the stack.
  The index must be a reference to an array of integers; likewise, the index into the array must be an integer. 
  Note:
  A NullPointerException is thrown if aref is null. An ArrayIndexOutOfBoundsException is thrown if the array index
  is not within the bounds of the array.
}}
{{
  JEM format: iaload
}}
  index := js.pop 'get index
  aref := js.pop  'get reference
  val := jd.readJavaIntBE(aref + 4 + (index<<1))
  js.push(val)    'push value
  jd.incJvmPC(1)
  

{{PUB jb_laload}}
{{ Meaning: Load long integer from array. }}
{{
  Description: The long integer value at the array aref is retrieved and pushed onto the top of the stack.
  The index must be a reference to an array of long integers; likewise, the index into the array must be an integer.
}}


{{PUB jb_faload}}
{{ Meaning: Load single float from array. }}
{{
  Description: The single-precision floating-point number at the array index is retrieved and pushed onto the
  top of the stack. The aref must be a reference to an array of single-precision floating-point numbers,
  and the index into the array must be an integer.
}}


{{PUB jb_daload}}
{{ Meaning: Load double float from array. }}
{{
  Description: The double-precision floating-point number at the array index is retrieved and pushed onto the
  top of the stack. The aref must be a reference to an array of double-precision floating-point numbers, and
  the index into the array must be an integer.
}}


{{PUB jb_aaload}}
{{ Meaning: Load object reference from array. }}
{{
  Description: The object reference at the array index is retrieved and pushed onto the top of the stack.
  The aref must be a reference to an array of object references, and the index into the array must be an integer.
}}


PUB jb_baload | index,val,aref
{{ Meaning: Load signed byte from array. }}
{{
  Description: The signed byte value at the array index is retrieved, expanded to an integer, and pushed onto
  the top of the stack. The aref must be a reference to an array of signed byte values, and the index into the
  array must be an integer.
}}
{{
  JEM format: baload
}}
  index := js.pop 'get index
  aref := js.pop  'get reference
  val := jd.readJavaByte(aref + 4 + index)
  js.push(val)    'push value
  jd.incJvmPC(1)


{{PUB jb_caload}}
{{ Meaning: Load character from array. }}
{{
  Description: The character value at the array index is retrieved, expanded to an integer, and pushed onto
  the top of the stack. The aref must be a reference to an array of character values, and the index into the
  array must be an integer.
}}


{{PUB jb_saload}}
{{ Meaning: Load short integer from array. }}
{{
  Description: The short integer value at the array index is retrieved, expanded to an integer, and pushed onto
  the top of the stack. The aref must be a reference to an array of short integer values and the index into the
  array must be an integer.
}}


PUB jb_iastore | val,index,aref
{{ Meaning: Store into integer array. }}
{{
  Description: An integer value is popped off the stack and stored in the array at the index. The aref must be
  a reference to an array of integer values, and the index into the array must be an integer as well.
}}
{{
  JEM format: iastore
}}
  val := js.pop   'get value
  index := js.pop 'get index
  aref := js.pop  'get reference
  jd.writeJavaIntBE(aref + 4 + (index<<1),val)
  jd.incJvmPC(1)


{{PUB jb_lastore}}
{{ Meaning: Store into long integer array. }}
{{
  Description: A long integer value is popped off the stack and stored in the array at the index. The aref must
  be a reference to an array of long integer values, and the index into the array must be an integer.
}}


{{PUB jb_fastore}}
{{ Meaning: Store into single float array. }}
{{
  Description: A single-precision floating-point number is popped off the stack and stored in the array at the index.
  The aref must be a reference to an array of single-precision floating-point numbers, and the index into the
  array must be an integer.
}}


{{PUB jb_dastore}}
{{ Meaning: Store into double float array. }}
{{
  Description: A double-precision floating-point number is popped off the stack and stored in the array at the index.
  The aref must be a reference to an array of double-precision floating-point numbers, and the index into the array
  must be an integer.
}}


{{PUB jb_aastore}}
{{ Meaning: Store into object reference array. }}
{{
  Description: An object reference is popped off the stack and stored in the array at the index. The aref must
  be a reference to an array of objects, and the index into the array must be an integer.
}}


PUB jb_bastore | val,index,aref
{{ Meaning: Store into signed byte array. }}
{{
  Description: An integer is popped off the stack, converted to a signed byte, and stored in the array at the index.
  The aref must be a reference to an array of signed bytes, and the index into the array must be an integer.
  If the integer value is too large to be a signed byte, it is truncated.
}}
{{
  JEM format: bastore
}}
  val := js.pop   'get value
  index := js.pop 'get index
  aref := js.pop  'get reference
  jd.writeJavaByte(aref + 4 + index,val)
  jd.incJvmPC(1)


{{PUB jb_castore}}
{{ Meaning: Store into character array. }}
{{
  Description: An integer is popped off the stack, converted to a character, and stored in the array at the index.
  The aref must be a reference to an array of characters and the index into the array must be an integer. If the
  integer value is too large to be a character, it is truncated.
}}


{{PUB jb_sastore}}
{{ Meaning: Store into short array. }}
{{
  Description: An integer is popped off the stack, converted to a short integer, and stored in the array at the index.
  The aref must be a reference to an array of short integers, and the index into the array must be an integer.
  If the integer value is too large to be a short integer, it is truncated.
}}
